# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class LocalizationIdQuery(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'elemental_ids': 'list[str]',
        'float_array': 'list[FloatArrayQuery]',
        'frame_state_ids': 'list[int]',
        'ids': 'list[int]',
        'media_ids': 'list[int]',
        'media_query': 'str',
        'object_search': 'AttributeOperationSpec',
        'state_ids': 'list[int]'
    }

    attribute_map = {
        'elemental_ids': 'elemental_ids',
        'float_array': 'float_array',
        'frame_state_ids': 'frame_state_ids',
        'ids': 'ids',
        'media_ids': 'media_ids',
        'media_query': 'media_query',
        'object_search': 'object_search',
        'state_ids': 'state_ids'
    }

    def __init__(self, elemental_ids=None, float_array=None, frame_state_ids=None, ids=None, media_ids=None, media_query=None, object_search=None, state_ids=None, local_vars_configuration=None):  # noqa: E501
        """LocalizationIdQuery - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._elemental_ids = None
        self._float_array = None
        self._frame_state_ids = None
        self._ids = None
        self._media_ids = None
        self._media_query = None
        self._object_search = None
        self._state_ids = None
        self.discriminator = None

        if elemental_ids is not None:
            self.elemental_ids = elemental_ids
        if float_array is not None:
            self.float_array = float_array
        if frame_state_ids is not None:
            self.frame_state_ids = frame_state_ids
        if ids is not None:
            self.ids = ids
        if media_ids is not None:
            self.media_ids = media_ids
        if media_query is not None:
            self.media_query = media_query
        if object_search is not None:
            self.object_search = object_search
        if state_ids is not None:
            self.state_ids = state_ids

    @property
    def elemental_ids(self):
        """
        Array of localizations to retrieve.

        :return: The elemental_ids of this LocalizationIdQuery. 
        :rtype: list[str]
        """
        return self._elemental_ids

    @elemental_ids.setter
    def elemental_ids(self, elemental_ids):
        """
        Array of localizations to retrieve.

        :param elemental_ids: The elemental_ids of this LocalizationIdQuery.
        :type: list[str]
        """

        self._elemental_ids = elemental_ids

    @property
    def float_array(self):
        """
        Searches on `float_array` attributes.

        :return: The float_array of this LocalizationIdQuery. 
        :rtype: list[FloatArrayQuery]
        """
        return self._float_array

    @float_array.setter
    def float_array(self, float_array):
        """
        Searches on `float_array` attributes.

        :param float_array: The float_array of this LocalizationIdQuery.
        :type: list[FloatArrayQuery]
        """

        self._float_array = float_array

    @property
    def frame_state_ids(self):
        """
        Array of frame state IDs for which localizations present in the same media/frame will be retrieved.

        :return: The frame_state_ids of this LocalizationIdQuery. 
        :rtype: list[int]
        """
        return self._frame_state_ids

    @frame_state_ids.setter
    def frame_state_ids(self, frame_state_ids):
        """
        Array of frame state IDs for which localizations present in the same media/frame will be retrieved.

        :param frame_state_ids: The frame_state_ids of this LocalizationIdQuery.
        :type: list[int]
        """

        self._frame_state_ids = frame_state_ids

    @property
    def ids(self):
        """
        Array of localization IDs to retrieve.

        :return: The ids of this LocalizationIdQuery. 
        :rtype: list[int]
        """
        return self._ids

    @ids.setter
    def ids(self, ids):
        """
        Array of localization IDs to retrieve.

        :param ids: The ids of this LocalizationIdQuery.
        :type: list[int]
        """

        self._ids = ids

    @property
    def media_ids(self):
        """
        Array of parent media IDs for which localizations should be retrieved.

        :return: The media_ids of this LocalizationIdQuery. 
        :rtype: list[int]
        """
        return self._media_ids

    @media_ids.setter
    def media_ids(self, media_ids):
        """
        Array of parent media IDs for which localizations should be retrieved.

        :param media_ids: The media_ids of this LocalizationIdQuery.
        :type: list[int]
        """

        self._media_ids = media_ids

    @property
    def media_query(self):
        """
        Query string used to filter media IDs. This can be used to avoid serialization and download of a media ID list.

        :return: The media_query of this LocalizationIdQuery. 
        :rtype: str
        """
        return self._media_query

    @media_query.setter
    def media_query(self, media_query):
        """
        Query string used to filter media IDs. This can be used to avoid serialization and download of a media ID list.

        :param media_query: The media_query of this LocalizationIdQuery.
        :type: str
        """

        self._media_query = media_query

    @property
    def object_search(self):
        """

        :return: The object_search of this LocalizationIdQuery. 
        :rtype: AttributeOperationSpec
        """
        return self._object_search

    @object_search.setter
    def object_search(self, object_search):
        """

        :param object_search: The object_search of this LocalizationIdQuery.
        :type: AttributeOperationSpec
        """

        self._object_search = object_search

    @property
    def state_ids(self):
        """
        Array of parent localization state (track) IDs for which localizations should be retrieved.

        :return: The state_ids of this LocalizationIdQuery. 
        :rtype: list[int]
        """
        return self._state_ids

    @state_ids.setter
    def state_ids(self, state_ids):
        """
        Array of parent localization state (track) IDs for which localizations should be retrieved.

        :param state_ids: The state_ids of this LocalizationIdQuery.
        :type: list[int]
        """

        self._state_ids = state_ids

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LocalizationIdQuery):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, LocalizationIdQuery):
            return True

        return self.to_dict() != other.to_dict()
