# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class UserUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'clear_avatar': 'int',
        'clear_profile_keys': 'list[str]',
        'elemental_id': 'str',
        'email': 'str',
        'first_name': 'str',
        'last_name': 'str',
        'new_avatar': 'str',
        'password': 'str',
        'reset_token': 'str',
        'set_profile_keys': 'object'
    }

    attribute_map = {
        'clear_avatar': 'clear_avatar',
        'clear_profile_keys': 'clear_profile_keys',
        'elemental_id': 'elemental_id',
        'email': 'email',
        'first_name': 'first_name',
        'last_name': 'last_name',
        'new_avatar': 'new_avatar',
        'password': 'password',
        'reset_token': 'reset_token',
        'set_profile_keys': 'set_profile_keys'
    }

    def __init__(self, clear_avatar=0, clear_profile_keys=None, elemental_id=None, email=None, first_name=None, last_name=None, new_avatar=None, password=None, reset_token=None, set_profile_keys=None, local_vars_configuration=None):  # noqa: E501
        """UserUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._clear_avatar = None
        self._clear_profile_keys = None
        self._elemental_id = None
        self._email = None
        self._first_name = None
        self._last_name = None
        self._new_avatar = None
        self._password = None
        self._reset_token = None
        self._set_profile_keys = None
        self.discriminator = None

        if clear_avatar is not None:
            self.clear_avatar = clear_avatar
        if clear_profile_keys is not None:
            self.clear_profile_keys = clear_profile_keys
        if elemental_id is not None:
            self.elemental_id = elemental_id
        if email is not None:
            self.email = email
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if new_avatar is not None:
            self.new_avatar = new_avatar
        if password is not None:
            self.password = password
        if reset_token is not None:
            self.reset_token = reset_token
        if set_profile_keys is not None:
            self.set_profile_keys = set_profile_keys

    @property
    def clear_avatar(self):
        """
        Set to >0 to clear the current avatar of the user

        :return: The clear_avatar of this UserUpdate. 
        :rtype: int
        """
        return self._clear_avatar

    @clear_avatar.setter
    def clear_avatar(self, clear_avatar):
        """
        Set to >0 to clear the current avatar of the user

        :param clear_avatar: The clear_avatar of this UserUpdate.
        :type: int
        """

        self._clear_avatar = clear_avatar

    @property
    def clear_profile_keys(self):
        """
        Clear the given keys from the profile dictionary

        :return: The clear_profile_keys of this UserUpdate. 
        :rtype: list[str]
        """
        return self._clear_profile_keys

    @clear_profile_keys.setter
    def clear_profile_keys(self, clear_profile_keys):
        """
        Clear the given keys from the profile dictionary

        :param clear_profile_keys: The clear_profile_keys of this UserUpdate.
        :type: list[str]
        """

        self._clear_profile_keys = clear_profile_keys

    @property
    def elemental_id(self):
        """
        Unique ID for a user across clusters/deployments

        :return: The elemental_id of this UserUpdate. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        Unique ID for a user across clusters/deployments

        :param elemental_id: The elemental_id of this UserUpdate.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def email(self):
        """
        Email address of user.

        :return: The email of this UserUpdate. 
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """
        Email address of user.

        :param email: The email of this UserUpdate.
        :type: str
        """

        self._email = email

    @property
    def first_name(self):
        """
        First name of user.

        :return: The first_name of this UserUpdate. 
        :rtype: str
        """
        return self._first_name

    @first_name.setter
    def first_name(self, first_name):
        """
        First name of user.

        :param first_name: The first_name of this UserUpdate.
        :type: str
        """

        self._first_name = first_name

    @property
    def last_name(self):
        """
        Last name of user.

        :return: The last_name of this UserUpdate. 
        :rtype: str
        """
        return self._last_name

    @last_name.setter
    def last_name(self, last_name):
        """
        Last name of user.

        :param last_name: The last_name of this UserUpdate.
        :type: str
        """

        self._last_name = last_name

    @property
    def new_avatar(self):
        """
        b64 encoded image to use for the avatar of the user.This file is verified to meet system constraints:    - Less than 1Mb     - a mimetype of image/png or image/jpg

        :return: The new_avatar of this UserUpdate. 
        :rtype: str
        """
        return self._new_avatar

    @new_avatar.setter
    def new_avatar(self, new_avatar):
        """
        b64 encoded image to use for the avatar of the user.This file is verified to meet system constraints:    - Less than 1Mb     - a mimetype of image/png or image/jpg

        :param new_avatar: The new_avatar of this UserUpdate.
        :type: str
        """

        self._new_avatar = new_avatar

    @property
    def password(self):
        """
        Account password.

        :return: The password of this UserUpdate. 
        :rtype: str
        """
        return self._password

    @password.setter
    def password(self, password):
        """
        Account password.

        :param password: The password of this UserUpdate.
        :type: str
        """

        self._password = password

    @property
    def reset_token(self):
        """
        Password reset token generated by PasswordReset object.

        :return: The reset_token of this UserUpdate. 
        :rtype: str
        """
        return self._reset_token

    @reset_token.setter
    def reset_token(self, reset_token):
        """
        Password reset token generated by PasswordReset object.

        :param reset_token: The reset_token of this UserUpdate.
        :type: str
        """

        self._reset_token = reset_token

    @property
    def set_profile_keys(self):
        """
        Key/Value Pairs to set atomically

        :return: The set_profile_keys of this UserUpdate. 
        :rtype: object
        """
        return self._set_profile_keys

    @set_profile_keys.setter
    def set_profile_keys(self, set_profile_keys):
        """
        Key/Value Pairs to set atomically

        :param set_profile_keys: The set_profile_keys of this UserUpdate.
        :type: object
        """

        self._set_profile_keys = set_profile_keys

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UserUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, UserUpdate):
            return True

        return self.to_dict() != other.to_dict()
